package bookmark.dao.jdbc;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import org.springframework.dao.DataAccessException;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.PreparedStatementCreator;
import org.springframework.jdbc.core.RowCallbackHandler;
import org.springframework.jdbc.core.support.JdbcDaoSupport;

import bookmark.dao.BookmarkDAO;
import bookmark.dao.DAOException;
import bookmark.model.Bookmark;

/**
 * ubN}[NpDAOJDBCɂB
 */
public class JDBCBookmarkDAO extends JdbcDaoSupport implements BookmarkDAO {

    public Bookmark retrieve(final Long id) throws DAOException {
        JdbcTemplate template = getJdbcTemplate();
        final String sql = "SELECT id,title,url FROM bookmark WHERE id = ?";
        final Bookmark bookmark = new Bookmark();

        try {
            template.query(
                new PreparedStatementCreator() {
                    public PreparedStatement createPreparedStatement(Connection con)
                            throws SQLException {
                        PreparedStatement stmt = con.prepareStatement(sql);
                        stmt.setLong(1, id.longValue());
                        return stmt;
                    }
                },
                new RowCallbackHandler() {
                    public void processRow(ResultSet rs) throws SQLException {
                        bookmark.setId(new Long(rs.getLong("id")));
                        bookmark.setTitle(rs.getString("title"));
                        bookmark.setUrl(rs.getString("url"));
                    }
                }
            );
        } catch (DataAccessException e) {
            throw new DAOException("Couldn't retrieve bookmark", e);
        }

        if (bookmark.getId() == null) {
            return null;
        }

        return bookmark;
    }

    public void create(final Bookmark bookmark) throws DAOException {
        JdbcTemplate template = getJdbcTemplate();
        final String sql = "INSERT INTO bookmark (title,url) VALUES (?,?)";

        try {
            template.update(
                new PreparedStatementCreator() {
                    public PreparedStatement createPreparedStatement(Connection con)
                            throws SQLException {
                        PreparedStatement stmt = con.prepareStatement(sql);
                        stmt.setString(1, bookmark.getTitle());
                        stmt.setString(2, bookmark.getUrl());
                        return stmt;
                    }
                }
            );
        } catch (DataAccessException e) {
            throw new DAOException("Couldn't create bookmark", e);
        }
    }

    public List retrieveAll() throws DAOException {
        JdbcTemplate template = getJdbcTemplate();
        final String sql = "SELECT id,title,url FROM bookmark ORDER BY id";
        final List bookmarks = new ArrayList();

        try {
            template.query(
                sql,
                new RowCallbackHandler() {
                    public void processRow(ResultSet rs) throws SQLException {
                        Bookmark bookmark = new Bookmark();
                        bookmark.setId(new Long(rs.getLong("id")));
                        bookmark.setTitle(rs.getString("title"));
                        bookmark.setUrl(rs.getString("url"));
                        bookmarks.add(bookmark);
                    }
                }
            );
        } catch (DataAccessException e) {
            throw new DAOException("Couldn't retrieve all bookmarks", e);
        }

        return bookmarks;
    }

}
